/**
 * $Id: MailboxListAction.java,v 1.1.2.1 2006/07/11 17:57:14 cvs Exp $
 * 쌠:       Copyright (c) 2006 ASKA Systems,Ltd. All right reserved.<p>
 * Ж:       ASKA Systems,Ltd.<p>
 */
package com.askasystems.MailAdmin;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.struts.actions.DispatchAction;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.validator.DynaValidatorForm;

import java.util.Iterator;
import java.util.regex.Pattern;
import java.util.List;

/**
 * [{bNXXgAction <BR>
 * 
 * @author Yasuhiro Yamamoto
 * @version 0.1
 */
public class MailboxListAction extends DispatchAction {

	static DomainManager pDomainManager_;

	static MailboxManager pMailboxManager_;

	static VirtualMapManager pVirtualMapManager_;

	static Logger pLogger_;

	static boolean pBCyradmAgent_;

	static boolean pBQuota_;

	private Pattern pvPattern = Pattern.compile("\\d+");

	public ActionForward in(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {

		DynaValidatorForm mLForm = (DynaValidatorForm) form;
		String sDomain = mLForm.getString("domain");

		HttpSession session = request.getSession();
		Ticket ticket = (Ticket) session.getAttribute("ticket");

		if (ticket == null)
			throw new SessionException();

		if (!ticket.isActive()
				|| (!ticket.isAdmin() && !ticket.getDomain().equals(sDomain)))
			throw new PermissionException();

		DomainInfo domainInfo = pDomainManager_.getDomainInfo(sDomain);
		List<MailboxInfo> lBoxes = pMailboxManager_.getMailboxList(sDomain);
		if (pBQuota_) {
			int[] nAmount = QuotaManager.setMailboxesUsed(lBoxes);
			QuotaManager.setDomainUsedString(domainInfo, nAmount);
		}

		session.setAttribute("boxes", lBoxes);
		session.setAttribute("domain_info", domainInfo);

		return mapping.findForward("box_list");

	}

	public ActionForward add(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {

		HttpSession session = request.getSession();
		Ticket ticket = (Ticket) session.getAttribute("ticket");

		DynaValidatorForm mLForm = (DynaValidatorForm) form;
		String sDomain = mLForm.getString("domain");

		if (ticket == null)
			throw new SessionException();

		if (!ticket.isActive()
				|| (!ticket.isAdmin() && !ticket.getDomain().equals(sDomain)))
			throw new PermissionException();

		DomainInfo domainInfo = pDomainManager_.getDomainInfo(sDomain);

		String sId = domainInfo.getPrefix() + mLForm.getString("id")
				+ domainInfo.getSuffix();
		String sPassword = mLForm.getString("password");
		String sName = mLForm.getString("name");
		String sQuota = mLForm.getString("quotableString");
		int nQuota = -1;
		if (pvPattern.matcher(sQuota).matches()) {
			nQuota = Integer.parseInt(sQuota);
		}

		if (pMailboxManager_.Exists(sId)) {

			ActionMessages actionMessages = new ActionMessages();
			actionMessages.add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.conflict", "[UID"));
			saveErrors(request, actionMessages);
			return mapping.findForward("box_list");
		}

		List<MailboxInfo> lBoxes = pMailboxManager_.getMailboxList(sDomain);
		if (pBQuota_) {
			int[] nAmount = QuotaManager.setMailboxesUsed(lBoxes);
			QuotaManager.setDomainUsedString(domainInfo, nAmount);

			if (domainInfo.getQuotable() == 0 || domainInfo.getQuotable() == -2) {
				ActionMessages actionMessages = new ActionMessages();
				actionMessages.add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.quota1"));
				saveErrors(request, actionMessages);
				return mapping.findForward("box_list");
			}

			if (domainInfo.getQuota() > -1
					&& (nQuota == -1 || nQuota > domainInfo.getQuotable())) {

				ActionMessages actionMessages = new ActionMessages();
				actionMessages.add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.quota2", domainInfo
								.getQuotableString()));
				saveErrors(request, actionMessages);
				return mapping.findForward("box_list");
			}
		}

		pMailboxManager_.addMailbox(sId, sPassword, sDomain, sName, nQuota);
		pLogger_.log(ticket, "add Mailbox", sId);
		if (pBCyradmAgent_) {
			pLogger_.log(ticket, "create Mailbox", CyradmAgent
					.createMailbox(sId));
			if (nQuota > -1)
				pLogger_.log(ticket, "set Quota", CyradmAgent.setQuota(
						sId, sQuota));
		}

		String sAddress = mLForm.getString("id");
		if (!domainInfo.isAdmin())
			sAddress += "@" + sDomain;
		pVirtualMapManager_.addAddress(sAddress, sDomain, sName, sId);

		lBoxes = pMailboxManager_.getMailboxList(sDomain);

		if (pBQuota_) {
			int[] nAmount = QuotaManager.setMailboxesUsed(lBoxes);
			QuotaManager.setDomainUsedString(domainInfo, nAmount);
		}

		session.setAttribute("boxes", lBoxes);
		session.setAttribute("domain_info", domainInfo);

		return mapping.findForward("box_list");

	}

	public ActionForward update(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {

		HttpSession session = request.getSession();
		Ticket ticket = (Ticket) session.getAttribute("ticket");

		DynaValidatorForm mLForm = (DynaValidatorForm) form;
		String sId = mLForm.getString("id");
		MailboxInfo boxInfo = pMailboxManager_.getMailboxInfo(sId);
		String sDomain = boxInfo.getDomain();

		if (ticket == null)
			throw new SessionException();

		if (!ticket.isActive()
				|| (!ticket.isAdmin() && !ticket.getDomain().equals(sDomain)))
			throw new PermissionException();

		DomainInfo domainInfo = pDomainManager_.getDomainInfo(sDomain);

		String sPassword = mLForm.getString("password");
		String sName = mLForm.getString("name");

		pMailboxManager_.updateMailbox(sId, sPassword, sName, boxInfo
				.getQuota());
		pLogger_.log(ticket, "update Mailbox", sId);

		List<MailboxInfo> lBoxes = pMailboxManager_.getMailboxList(sDomain);
		if (pBQuota_) {
			int[] nAmount = QuotaManager.setMailboxesUsed(lBoxes);
			QuotaManager.setDomainUsedString(domainInfo, nAmount);
		}

		session.setAttribute("boxes", lBoxes);
		session.setAttribute("domain_info", domainInfo);

		return mapping.findForward("box_list");

	}

	public ActionForward setQuota(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {

		HttpSession session = request.getSession();
		Ticket ticket = (Ticket) session.getAttribute("ticket");

		DynaValidatorForm mLForm = (DynaValidatorForm) form;
		String sId = mLForm.getString("id");
		MailboxInfo boxInfo = pMailboxManager_.getMailboxInfo(sId);
		String sDomain = boxInfo.getDomain();

		if (ticket == null)
			throw new SessionException();

		if (!ticket.isActive()
				|| (!ticket.isAdmin() && !ticket.getDomain().equals(sDomain)))
			throw new PermissionException();

		DomainInfo domainInfo = pDomainManager_.getDomainInfo(sDomain);

		String sQuota = mLForm.getString("quotaString").trim();
		int nQuota = -1;
		if (pvPattern.matcher(sQuota).matches()) {
			nQuota = Integer.parseInt(sQuota);
		}

		List<MailboxInfo> lBoxes = pMailboxManager_.getMailboxList(sDomain);
		if (pBQuota_) {
			int[] nAmount = QuotaManager.setMailboxesUsed(lBoxes);
			QuotaManager.setDomainUsedString(domainInfo, nAmount);
		}

		int nCQuota = boxInfo.getQuota();
		if (nCQuota == -1)
			nCQuota = 0;
		if ((domainInfo.getQuota() > -1)
				&& (nQuota == -1 || (nQuota > domainInfo.getQuotable()
						+ nCQuota))) {

			ActionMessages actionMessages = new ActionMessages();
			actionMessages.add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.quota", domainInfo
							.getQuotableString()));
			saveErrors(request, actionMessages);
			return mapping.findForward("box_list");
		}

		pMailboxManager_.updateMailbox(sId, boxInfo.getPassword(), boxInfo
				.getName(), nQuota);
		pLogger_.log(ticket, "update Quota", sId);
		if (pBCyradmAgent_) {
			if (nQuota == -1)
				sQuota = "none";
			pLogger_.log(ticket, "set Quota", CyradmAgent.setQuota(sId,
					sQuota));
		}

		if (pBQuota_) {
			lBoxes = pMailboxManager_.getMailboxList(sDomain);
			int[] nAmount = QuotaManager.setMailboxesUsed(lBoxes);
			QuotaManager.setDomainUsedString(domainInfo, nAmount);
		}

		session.setAttribute("boxes", lBoxes);
		session.setAttribute("domain_info", domainInfo);

		return mapping.findForward("box_list");

	}

	public ActionForward delete(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {

		HttpSession session = request.getSession();
		Ticket ticket = (Ticket) session.getAttribute("ticket");

		DynaValidatorForm mLForm = (DynaValidatorForm) form;
		String sId = mLForm.getString("id");
		MailboxInfo boxInfo = pMailboxManager_.getMailboxInfo(sId);
		String sDomain = boxInfo.getDomain();

		if (ticket == null)
			throw new SessionException();

		if (!ticket.isActive()
				|| (!ticket.isAdmin() && !ticket.getDomain().equals(sDomain)))
			throw new PermissionException();

		pMailboxManager_.removeMailbox(sId);
		pLogger_.log(ticket, "remove Mailbox", sId + " removed");
		if (pBCyradmAgent_) {
			pLogger_.log(ticket, "deleteMailbox", CyradmAgent
					.deleteMailbox(sId));
		}

		Iterator<AddressInfo> i = pVirtualMapManager_.getAddressList(sDomain)
				.iterator();
		while (i.hasNext()) {
			AddressInfo addressInfo = i.next();
			pVirtualMapManager_.removeMailbox(addressInfo.getAddress(),
					sId);
		}

		DomainInfo domainInfo = pDomainManager_.getDomainInfo(sDomain);
		List<MailboxInfo> lBoxes = pMailboxManager_.getMailboxList(sDomain);
		if (pBQuota_) {
			int[] nAmount = QuotaManager.setMailboxesUsed(lBoxes);
			QuotaManager.setDomainUsedString(domainInfo, nAmount);
		}

		session.setAttribute("boxes", lBoxes);
		session.setAttribute("domain_info", domainInfo);

		return mapping.findForward("box_list");
	}

	public ActionForward changeActive(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {

		HttpSession session = request.getSession();
		Ticket ticket = (Ticket) session.getAttribute("ticket");

		DynaValidatorForm mLForm = (DynaValidatorForm) form;
		String sId = mLForm.getString("id");
		MailboxInfo boxInfo = pMailboxManager_.getMailboxInfo(sId);
		String sDomain = boxInfo.getDomain();

		if (ticket == null)
			throw new SessionException();

		if (!ticket.isActive()
				|| (!ticket.isAdmin() && !ticket.getDomain().equals(sDomain)))
			throw new PermissionException();

		boolean bActive = (Boolean) (mLForm.get("active"));

		pMailboxManager_.changeActive(sId, bActive);
		pLogger_.log(ticket, "change Active of Mailbox", sId + " ("
				+ bActive + ")");

		DomainInfo domainInfo = pDomainManager_.getDomainInfo(sDomain);
		List<MailboxInfo> lBoxes = pMailboxManager_.getMailboxList(sDomain);
		if (pBQuota_) {
			int[] nAmount = QuotaManager.setMailboxesUsed(lBoxes);
			QuotaManager.setDomainUsedString(domainInfo, nAmount);
		}

		session.setAttribute("boxes", lBoxes);
		session.setAttribute("domain_info", domainInfo);

		return mapping.findForward("box_list");
	}

}
