/**
 * $Id: MySQLDomainManager.java,v 1.1.2.1 2006/07/11 17:57:14 cvs Exp $
 * 쌠:       Copyright (c) 2006 ASKA Systems,Ltd. All right reserved.<p>
 * Ж:       ASKA Systems,Ltd.<p>
 */
package com.askasystems.MailAdmin;

import java.sql.Connection;
import java.sql.Timestamp;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.util.List;
import java.util.LinkedList;

import javax.sql.DataSource;

public class MySQLDomainManager implements DomainManager {

	static DataSource pDataSource_;

	static String pSAdminDomain_ = "admin";

	public void addDomain(String sDomain, String sDescription, String sPrefix,
			String sSuffix, int nQuota) {

		try {

			Connection con = pDataSource_.getConnection();
			String sSql = "insert into domains (domain, description, prefix, suffix, quota, created, modified) values (?, ?, ?, ?, ?, ?, ?)";
			PreparedStatement st = con.prepareStatement(sSql);
			st.setString(1, sDomain);
			st.setString(2, sDescription);
			st.setString(3, sPrefix);
			st.setString(4, sSuffix);
			st.setInt(5, nQuota);
			st.setTimestamp(6, new Timestamp((new java.util.Date().getTime())));
			st.setTimestamp(7, new Timestamp((new java.util.Date().getTime())));
			st.executeUpdate();

			st.close();
			con.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}

	}

	public void removeDomain(String sDomain) {

		try {

			Connection con = pDataSource_.getConnection();
			String sSql = "delete from domains where domain = ?";
			PreparedStatement st = con.prepareStatement(sSql);
			st.setString(1, sDomain);
			st.executeUpdate();

			st.close();
			con.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}

	}

	public void updateDomain(String sDomain, String sDescription, int nQuota) {

		try {
			Connection con = pDataSource_.getConnection();
			String sSql = "update domains set description = ?, quota = ?, modified = ? where domain = ?";
			PreparedStatement st = con.prepareStatement(sSql);
			st.setString(1, sDescription);
			st.setInt(2, nQuota);
			st.setTimestamp(3, new Timestamp((new Date().getTime())));
			st.setString(4, sDomain);
			st.executeUpdate();

			st.close();
			con.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
	}

	public void changeActive(String sDomain, boolean bActive) {

		try {
			Connection con = pDataSource_.getConnection();
			String sSql = "update domains set active = ?,  modified = ? where domain = ?";
			PreparedStatement st = con.prepareStatement(sSql);
			st.setBoolean(1, bActive);
			st.setTimestamp(2, new Timestamp((new Date().getTime())));
			st.setString(3, sDomain);
			st.executeUpdate();

			st.close();
			con.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
	}

	public List<DomainInfo> getDomainList() {

		LinkedList<DomainInfo> lkLsDomain = new LinkedList<DomainInfo>();

		try {

			Connection con = pDataSource_.getConnection();
			String sSql = "select * from domains order by domain";
			PreparedStatement st = con.prepareStatement(sSql);
			ResultSet rtSet = st.executeQuery();

			while (rtSet.next()) {

				DomainInfo domainInfo = new DomainInfo();

				domainInfo.pSDomain = rtSet.getString("domain");
				domainInfo.pSDescription = rtSet.getString("description");
				domainInfo.pSPrefix = rtSet.getString("prefix");
				domainInfo.pSSuffix = rtSet.getString("suffix");
				domainInfo.pNQuota = rtSet.getInt("quota");
				domainInfo.pBActive = rtSet.getBoolean("active");
				domainInfo.pDateCreated = rtSet.getTimestamp("created");
				domainInfo.pDateModified = rtSet.getTimestamp("modified");
				if (domainInfo.pSDomain.equals(pSAdminDomain_))
					domainInfo.pBAdmin = true;

				lkLsDomain.add(domainInfo);

			}

			st.close();
			con.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}

		return lkLsDomain;
	}

	public DomainInfo getDomainInfo(String sDomain) {

		DomainInfo domainInfo = null;

		try {

			Connection con = pDataSource_.getConnection();
			String sSql = "select * from domains where domain = ?";
			PreparedStatement st = con.prepareStatement(sSql);
			st.setString(1, sDomain);
			ResultSet rtSet = st.executeQuery();

			while (rtSet.next()) {

				domainInfo = new DomainInfo();

				domainInfo.pSDomain = rtSet.getString("domain");
				domainInfo.pSDescription = rtSet.getString("description");
				domainInfo.pSPrefix = rtSet.getString("prefix");
				domainInfo.pSSuffix = rtSet.getString("suffix");
				domainInfo.pNQuota = rtSet.getInt("quota");
				domainInfo.pBActive = rtSet.getBoolean("active");
				domainInfo.pDateCreated = rtSet.getTimestamp("created");
				domainInfo.pDateModified = rtSet.getTimestamp("modified");
				if (domainInfo.pSDomain.equals(pSAdminDomain_))
					domainInfo.pBAdmin = true;

			}

			st.close();
			con.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}

		return domainInfo;
	}

	public boolean Exists(String sDomain) {

		boolean b = false;

		try {
			Connection con = pDataSource_.getConnection();
			String sSql = "select * from domains where domain = ?";
			PreparedStatement st = con.prepareStatement(sSql);
			st.setString(1, sDomain);
			ResultSet rtSet = st.executeQuery();
			b = rtSet.next();

			st.close();
			con.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}

		return b;

	}

	public boolean noAdmin() {

		return !Exists(pSAdminDomain_);
	}

}
