/**
 * $Id: MySQLMailboxManager.java,v 1.1.2.1 2006/07/11 17:57:14 cvs Exp $
 * 쌠:       Copyright (c) 2006 ASKA Systems,Ltd. All right reserved.<p>
 * Ж:       ASKA Systems,Ltd.<p>
 */
package com.askasystems.MailAdmin;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import javax.sql.DataSource;

public class MySQLMailboxManager implements MailboxManager {

	static DataSource pDataSource_;
	
	public void addMailbox(String sId, String sPassword, String sDomain,
			String sName, int nQuota) {
		
		try {

			Connection con = pDataSource_.getConnection();
			String sSql = "insert into mailboxes (id, domain, password, name, quota, created, modified) values (?, ?, ?, ?, ?, ?, ?)";
			PreparedStatement st = con.prepareStatement(sSql);
			st.setString(1, sId);
			st.setString(2, sDomain);
			st.setString(3, sPassword);
			st.setString(4, sName);
			st.setInt(5, nQuota) ;
			st.setTimestamp(6, new Timestamp((new java.util.Date().getTime())));
			st.setTimestamp(7, new Timestamp((new java.util.Date().getTime())));
			st.executeUpdate();

			st.close();
			con.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}

	}

	public void removeMailbox(String sId) {

		try {

			Connection con = pDataSource_.getConnection();
			String sSql = "delete from mailboxes where id = ?";
			PreparedStatement st = con.prepareStatement(sSql);
			st.setString(1, sId);
			st.executeUpdate();

			st.close();
			con.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}

	}

	public void updateMailbox(String sId, String sPassword, String sName, int nQuota) {

		try {
			Connection con = pDataSource_.getConnection();
			String sSql = "update mailboxes set password = ?, name = ?, modified = ?, quota = ? where id = ?";
			PreparedStatement st = con.prepareStatement(sSql);
			st.setString(1, sPassword);
			st.setString(2, sName);
			st.setTimestamp(3, new Timestamp((new Date().getTime())));
			st.setInt(4, nQuota) ;
			st.setString(5, sId);
			st.executeUpdate();

			st.close();
			con.close();
			
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
		
	}

	public void changeActive(String sId, boolean bActive) {
		
		try {
			Connection con = pDataSource_.getConnection();
			String sSql = "update mailboxes set active = ?, modified = ? where id = ?";
			PreparedStatement st = con.prepareStatement(sSql);
			st.setBoolean(1, bActive);
			st.setTimestamp(2, new Timestamp((new Date().getTime())));
			st.setString(3, sId);
			st.executeUpdate();

			st.close();
			con.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}

	}

	public List<MailboxInfo> getMailboxList(String sDomain) {
		
		LinkedList<MailboxInfo> lkLsBox = new LinkedList<MailboxInfo>();

		try {

			Connection con = pDataSource_.getConnection();
			String sSql = "select * from mailboxes where domain = ? order by id";
			PreparedStatement st = con.prepareStatement(sSql);
			st.setString(1, sDomain);
			ResultSet rtSet = st.executeQuery();

			while (rtSet.next()) {

				MailboxInfo boxInfo = new MailboxInfo();

				boxInfo.pSId = rtSet.getString("id");
				boxInfo.pSDomain = rtSet.getString("domain");
				boxInfo.pSPassword = rtSet.getString("password");
				boxInfo.pSName = rtSet.getString("name");
				boxInfo.pNQuota = rtSet.getInt("quota") ;
				boxInfo.pBActive = rtSet.getBoolean("active") ;
				boxInfo.pDateCreated = rtSet.getTimestamp("created");
				boxInfo.pDateModified = rtSet.getTimestamp("modified");

				lkLsBox.add(boxInfo);
			}

			st.close();
			con.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}

		return lkLsBox;
	}

	public MailboxInfo getMailboxInfo(String sId) {
	
		MailboxInfo boxInfo = null;

		try {

			Connection con = pDataSource_.getConnection();
			String sSql = "select * from mailboxes where id = ?";
			PreparedStatement st = con.prepareStatement(sSql);
			st.setString(1, sId);
			ResultSet rtSet = st.executeQuery();

			while (rtSet.next()) {

				boxInfo = new MailboxInfo();

				boxInfo.pSId = rtSet.getString("id");
				boxInfo.pSDomain = rtSet.getString("domain");
				boxInfo.pSPassword = rtSet.getString("password");
				boxInfo.pSName = rtSet.getString("name");
				boxInfo.pNQuota = rtSet.getInt("quota") ;
				boxInfo.pBActive = rtSet.getBoolean("active") ;
				boxInfo.pDateCreated = rtSet.getTimestamp("created");
				boxInfo.pDateModified = rtSet.getTimestamp("modified");
			}

			st.close();
			con.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}

		return boxInfo;
	}

	public boolean Exists(String sId) {

		boolean b = false;

		try {
			Connection con = pDataSource_.getConnection();
			String sSql = "select * from mailboxes where id = ?";
			PreparedStatement st = con.prepareStatement(sSql);
			st.setString(1, sId);
			ResultSet rtSet = st.executeQuery();
			b = rtSet.next();

			st.close();
			con.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}

		return b;
	}

	public int getMailBoxNumber(String sDomain) {

		int nNumber = 0;

		try {
			Connection con = pDataSource_.getConnection();
			String sSql = "select count(*) from mailboxes where domain = ?";
			PreparedStatement st = con.prepareStatement(sSql);
			st.setString(1, sDomain);
			ResultSet rtSet = st.executeQuery();
			rtSet.next();
			nNumber = rtSet.getInt(1);

			st.close();
			con.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}

		return nNumber;
	}

}
